RawVectorMapper

Struct RawVectorMapper 

pub struct RawVectorMapper<'a> { /* private fields */ }
An immutable memory-mapped RawVector.

This is compatible with the serialization format of RawVector.

Examples

use simple_sds_sbwt::raw_vector::{RawVector, RawVectorMapper, AccessRaw, PushRaw};
use simple_sds_sbwt::serialize::{MemoryMap, MemoryMapped, MappingMode};
use simple_sds_sbwt::serialize;
use std::fs;

let filename = serialize::temp_file_name("raw-vector-mapper");
let width = 29;
let mut original = RawVector::new();
unsafe {
    original.push_int(123, width);
    original.push_int(456, width);
    original.push_int(789, width);
}
serialize::serialize_to(&original, &filename);

let map = MemoryMap::new(&filename, MappingMode::ReadOnly).unwrap();
let mapper = RawVectorMapper::new(&map, 0).unwrap();
assert_eq!(mapper.len(), 3 * width);
unsafe {
    assert_eq!(mapper.int(0, width), 123);
    assert_eq!(mapper.int(width, width), 456);
    assert_eq!(mapper.int(2 * width, width), 789);
}

drop(mapper); drop(map);
fs::remove_file(&filename);

Implementations

Source

impl<'a> RawVectorMapper<'a>

pub fn len(&self) -> usize

Returns the length of the vector in bits.

pub fn is_empty(&self) -> bool

Returns true if the vector is empty.

pub fn count_ones(&self) -> usize

Counts the number of ones in the bit array.