MappedOption

Struct MappedOption 

pub struct MappedOption<'a, T: MemoryMapped<'a>> { /* private fields */ }
An optional immutable memory-mapped structure.

This is compatible with the serialization format of Option of the same type.

Examples

use simple_sds_sbwt::serialize::{MappedOption, MappedSlice, MappingMode, MemoryMap, MemoryMapped, Serialize};
use simple_sds_sbwt::serialize;
use std::fs;

let some: Option<Vec<u64>> = Some(vec![123, 456, 789]);
let filename = serialize::temp_file_name("mapped-option");
serialize::serialize_to(&some, &filename);

let map = MemoryMap::new(&filename, MappingMode::ReadOnly).unwrap();
let mapped = MappedOption::<MappedSlice<u64>>::new(&map, 0).unwrap();
assert_eq!(mapped.unwrap().as_ref(), some.unwrap().as_slice());
drop(mapped); drop(map);

fs::remove_file(&filename).unwrap();

Implementations

Source

impl<'a, T: MemoryMapped<'a>> MappedOption<'a, T>

pub fn is_some(&self) -> bool

Returns true if the option is a Some value.

pub fn is_none(&self) -> bool

Returns true if the option is a None value.

pub fn unwrap(&self) -> &T

Returns an immutable reference to the possibly contained value.

Panics

Panics if the option is a None value.

pub fn as_ref(&self) -> Option<&T>

Returns Option<&T> referencing the possibly contained value.