Serializable

Trait Serializable 

pub trait Serializable: Sized + Default {
    // Provided method
    fn elements() -> usize { ... }
}
A fixed-size type that can be serialized as one or more u64 elements.

Provided Methods

fn elements() -> usize

Returns the number of elements needed for serializing the type.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.

Implementations on Foreign Types

Source

impl Serializable for (u64, u64)

Source

impl Serializable for u64

Source

impl Serializable for usize

Implementors