pub struct RankSupport { /* private fields */ }Expand description
Rank support structure for plain bitvectors.
The structure depends on the parent bitvector and assumes that the parent remains unchanged.
Using the BitVector interface is usually more convenient.
Implementations
Sourceimpl RankSupport
impl RankSupport
Sourcepub const BLOCK_SIZE: usize = 512usize
pub const BLOCK_SIZE: usize = 512usize
Number of bits per block (512).
Sourcepub fn new(parent: &BitVector) -> RankSupport
pub fn new(parent: &BitVector) -> RankSupport
Builds a rank support structure for the parent bitvector.
Examples
use simple_sds_sbwt::bit_vector::BitVector;
use simple_sds_sbwt::bit_vector::rank_support::RankSupport;
let data = vec![false, true, true, false, true, false, true, true, false, false, false];
let bv: BitVector = data.into_iter().collect();
let rs = RankSupport::new(&bv);
assert_eq!(rs.blocks(), 1);Sourcepub fn rank(&self, parent: &BitVector, index: usize) -> usize
pub fn rank(&self, parent: &BitVector, index: usize) -> usize
Returns the rank at the specified index of the bitvector.
Arguments
parent: The parent bitvector.index: Index in the bit array or value in the integer array.
Examples
use simple_sds_sbwt::bit_vector::BitVector;
use simple_sds_sbwt::bit_vector::rank_support::RankSupport;
let data = vec![false, true, true, false, true, false, true, true, false, false, false];
let bv: BitVector = data.into_iter().collect();
let rs = RankSupport::new(&bv);
assert_eq!(rs.rank(&bv, 0), 0);
assert_eq!(rs.rank(&bv, 1), 0);
assert_eq!(rs.rank(&bv, 2), 1);
assert_eq!(rs.rank(&bv, 7), 4);Panics
May panic if index >= parent.len().
Sourcepub unsafe fn rank_unchecked(&self, parent: &BitVector, index: usize) -> usize
pub unsafe fn rank_unchecked(&self, parent: &BitVector, index: usize) -> usize
Unsafe version of RankSupport::rank without bounds checks.
Safety
Behavior is undefined if index >= parent.len().